/*
  ESP8266 Demo
  WIFI Network Scan
  Display: 128x64 OLED SSD1306 Controller I2C
  ESP8266 Connect:
  - D1 (GPIO5) --- SCL
  - D2 (GPIO4) --- SDA
*/

// https://blog.robberg.net/wifi-channel-scanner-with-wemos-d1-mini/



#include <Wire.h> // I2C lib
#include "oled_ssd1306.h"
OLED oled;

#define DISPMSGLENGTH 32
char displayMsg[DISPMSGLENGTH];

#include "ESP8266WiFi.h"
WiFiClient client;

#define SSIDLENGTH 32
int numNetworks;
int8_t sortWifi[99];


void demoHello() {
  oled.displayFill(0x00);  //  cls
  oled.displayFormat(2, oled.NORMAL);
  oled.displayPrintXY(0, 1, F("<   WiFiScanOled2   >"));
  oled.displayPrintXY(0, 5, F("<    2018-10-19a    >"));
}


void setup() {
  // initialize serial communication:
  Serial.begin(115200);
  Serial.println("\n\rStart");
  pinMode(LED_BUILTIN, OUTPUT);     // Initialize the LED_BUILTIN pin as an output
  digitalWrite(LED_BUILTIN, LOW);   // Turn the LED on
  delay(100);
  digitalWrite(LED_BUILTIN, HIGH);  // Turn the LED off
  
  // I2C Init
  Wire.begin();
  //Wire.setClock(400000); // 400kHz I2C Speed
  Wire.setClock(800000); // 800kHz I2C Speed
  //TWBR=IIC400KHZCLOCK; // 400kHz I2C Speed (AVR 8 Bit MCU)
  
  // OLED
  oled.D1306_begin(D1306_I2CADDRESS);  // initialize with the I2C addr
  delay(1000);
  oled.displayFill(0x00);  //  cls
  
  // Scroll large Message
//  oled._charStretchX = 3;
//  oled._charStretchY = 4;
  oled.displayScale(3,4);
  oled.scrollText("WiFiScanOled2",2);
  oled.displayFill(0x00);
  
  // hello screen
  delay(1000);
  demoHello();
  delay(3000);
  oled.displayFill(0x00);  //  cls
}


void loop() {
  
  ///////////////////////////////////////
  //
  // SCAN
  //
  ///////////////////////////////////////
  
  oled.displayFormat(3, oled.NORMAL);
  //displayFill(0x00);  //  OLED - cls
  //displayClrLine(0);
  
  //SSID_scan();
  WiFi.disconnect();
  delay(100);
  //displayClrLine(0);
  oled.displayPrintXY(8*12, 0, F("->"));
  
  Serial.print("SSID_scan ...");
//  digitalWrite(LED_BUILTIN, LOW);   // Turn the LED on
//  numNetworks = WiFi.scanNetworks();
  numNetworks = WiFi.scanNetworks(false,true);
  oled.displayFill(0x00);  //  OLED - cls
//  digitalWrite(LED_BUILTIN, HIGH);  // Turn the LED off
  Serial.println(" done.");
  
  ///////////////////////////////////////
  //
  // Sort Networks
  //
  ///////////////////////////////////////
  
  // step 1 - copy indices
  uint8_t maxWifi = numNetworks;
  if (maxWifi>99) maxWifi = 99;
  for(int i=0; i<maxWifi; i++) {
    sortWifi[i]=i;
  }
  
  // step 2 - bubblesort indices
  for(int i=0; i<(maxWifi-1); i++) {
    for(int j=i+1; j<maxWifi; j++) {
      if(WiFi.RSSI(sortWifi[j]) > WiFi.RSSI(sortWifi[i])) {
        int8_t save = sortWifi[i];
        sortWifi[i] = sortWifi[j];
        sortWifi[j] = save;
      }
    }
  }
  
  ///////////////////////////////////////
  //
  // Show Networks
  //
  ///////////////////////////////////////
  
//  displayFill(0x00);  //  OLED - cls
  
  oled.displayFormat(3, oled.NORMAL);
  oled.displayClrLine(0);
  oled.displayXYUInt(0, 0, numNetworks, 2);
  oled.displayPrintXY(3*12, 0, F("W"));
  
  // count open networks
  uint8_t openWlans = 0;
  for (int i = 0; i < numNetworks; i++) {
    if(WiFi.encryptionType(i) == ENC_TYPE_NONE) {
      openWlans++;
    }
  }
  // print open networks
  if (openWlans != 0) {
    oled.displayPrintXY(5*12, 0, F("*"));
    oled.displayXYUInt(6*12, 0, openWlans, 1);
  }
  
  showNetworks();
//  show3NetworksOled();
//  show6NetworksOled();
//  delay(2000);
  
  char s[SSIDLENGTH];
  char sList[SSIDLENGTH*10];
  oled.displayFormat(3, oled.NORMAL);
  if(numNetworks==0){
  } else {
    uint8_t strongest = 10;
    if (numNetworks < strongest) strongest = numNetworks;
    if(strongest>0) {
      sList[0] = 0;   // clear list
      for (int i = 0; i < strongest; i++) {
        if(WiFi.encryptionType(sortWifi[i]) == ENC_TYPE_NONE) {
          strcat(sList, "(*)");
          // open networks start with "(*)"
        }
        WiFi.SSID(sortWifi[i]).toCharArray(s, SSIDLENGTH);
        strcat(sList, s);
        strcat(sList, " .. ");
      }
      oled.displayScale(3,4);
      oled.scrollTextFast(sList,4);
    }
  }
//  show3NetworksOled();
  show6NetworksOled();
}


/////////////////////////////////////////////////////////
//
// WIFI Scan
//
/////////////////////////////////////////////////////////


void SSID_scan() {
  Serial.println("");
  Serial.println("scan start");
  WiFi.disconnect();
  delay(100);
  // WiFi.scanNetworks will return the number of networks found
  numNetworks = WiFi.scanNetworks();
  if (numNetworks == 0)
    Serial.println("no networks found");
  else
  {
    Serial.print(numNetworks);
    Serial.println(" networks found");
    for (int i = 0; i < numNetworks; ++i)
    {
      // Print SSID and RSSI for each network found
      Serial.print(i + 1);
      Serial.print(": ");
      Serial.print(WiFi.SSID(i));
      Serial.print(" (");
      Serial.print(WiFi.RSSI(i));
      Serial.print(") ");
      Serial.print(" [");
      Serial.print(WiFi.channel(i));
      Serial.print("] ");
      Serial.println((String) encryptionTypeStr(WiFi.encryptionType(i)));
      delay(10);
    }
  }
  Serial.println("scan done");
  Serial.println("");
}

void showNetworks() {
  // sortWifi[i]
  Serial.println();
  Serial.println("WiFi Networks");
  if (numNetworks == 0)
    Serial.println("no networks found");
  else
  {
    Serial.print(numNetworks);
    Serial.println(" networks found");
    for (int i = 0; i < numNetworks; ++i)
    {
      // Print SSID and RSSI for each network found
      Serial.print(i + 1);
      Serial.print(": ");
      Serial.print(WiFi.SSID(sortWifi[i]));
      Serial.print(" (");
      Serial.print(WiFi.RSSI(sortWifi[i]));
      Serial.print(") ");
      Serial.print(" [");
      Serial.print(WiFi.channel(sortWifi[i]));
      Serial.print("] ");
      Serial.println((String) encryptionTypeStr(WiFi.encryptionType(sortWifi[i])));
      delay(10);
    }
  }
  Serial.println();
}


// print first 3 networks (12x16 font), less details
void show3NetworksOled() {
  char s[21];
  oled.displayFormat(3, oled.NORMAL);
  for (int i = 0; i < 3; i++) {
    oled.displayClrLine(2+(2*i));
    if (i < numNetworks) {
      if(WiFi.encryptionType(sortWifi[i]) == ENC_TYPE_NONE) {
        oled.displayPrintXY(0, 2+(2*i), F("*"));
      } else {
        oled.displayPrintXY(0, 2+(2*i), F("~"));
      }
      WiFi.SSID(sortWifi[i]).toCharArray(s, 10);
      oled.displayPrintXY(12, 2+(2*i), s);
    } else {
      oled.displayPrintXY(0, 2+(2*i), F(" - - - - -"));
    }
  }
}


// print first 6 networks (6x8 font), more details
void show6NetworksOled() {
  char s[21];
  oled.displayFormat(1, oled.NORMAL);
  for (int i = 0; i < 6; i++) {
    oled.displayClrLine(2+i);
    if (i < numNetworks) {
      // signal strength (RSSI)
      oled.displayPrintXY(0, 2+i, F("-"));
      oled.displayXYUInt(6, 2+i, -WiFi.RSSI(sortWifi[i]), 2);
      // "*" = open, no encryption
      if(WiFi.encryptionType(sortWifi[i]) == ENC_TYPE_NONE) {
        oled.displayPrintXY(3*6, 2+i, F("*"));
      }
      // Network Name (SSID)
      WiFi.SSID(sortWifi[i]).toCharArray(s, 18);
      oled.displayPrintXY(4*6, 2+i, s);
    } else {
      oled.displayPrintXY(0, 2+i, F("- none -"));
    }
  }
}


String encryptionTypeStr(uint8_t authmode) {
  switch (authmode) {
    case ENC_TYPE_NONE:
      return "NONE";
    case ENC_TYPE_WEP:
      return "WEP";
    case ENC_TYPE_TKIP:
      return "WPA";
    case ENC_TYPE_CCMP:
      return "WPA2";
    case ENC_TYPE_AUTO:
      return "AUTO";
    default:
      return "?";;
  }
}

