/* OLED SSD 1306 (128x64) IIC-Interface library 

License: GPL V3
written by Sven K. (oog / proog.de)

History:
YYYY-MM-DD
2018-10:
  first lib-version
2018-10-19:
  Print text within screen range
  now scrolls without buffer

*/


#ifndef OLEDSSD1306_H
#define OLEDSSD1306_H
#include "Arduino.h"

// choose a font for the display print statement
#define fnt6x8
//#define fnt8x8

// 400kHz I2C Speed (AVR 8 Bit MCU)
#define IIC400KHZCLOCK 12

// I2C OLED Display
#define D1306_I2CADDRESS 0x3C
#define D1306_WIDTH 128
#define D1306_HEIGHT 64

class OLED {
  private:
  uint8_t oledOk = 1;
  uint8_t _i2caddr;
  uint8_t _displayCursX;
  uint8_t _displayCursY;
  // global buffer for 16 Bytes Graphics
  // _buf16[0] = byte counter 1..16
  uint8_t _buf16[17];
  
  public:
  void D1306_cmd(uint8_t c);
  void D1306_cmd(uint8_t c, uint8_t d);
  void D1306_begin(uint8_t i2caddr);
  int strPix(char s[]);
  void displayCursXY(uint8_t x, uint8_t y);
  void displayFill(uint8_t d);
  void displayFill(uint8_t d1,uint8_t d2);
  void displayByte(uint8_t d);
  void displayContrast(uint8_t d);

  uint8_t charLineY=1;
  
  uint8_t scaleX=1;
  uint8_t scaleY=1;
  uint8_t charMask=0;
  const uint8_t NORMAL=0;
  const uint8_t INVERSE=1;
  const uint8_t UNDERLINE=2;
  
  //
  // Direct print to Display
  //
  void displayBuf(void);
  void displayBufN(uint8_t ptr[], uint8_t count);
  void displayRomChar(int startX, int endX, uint8_t c);
  
  void displayFormat(uint8_t fntsize, uint8_t format);
  void displayScale(uint8_t scale_X, uint8_t scale_Y);
  void dispOneLine(int px,int py,int xs,int xe,int firstchar,int lastchar,char s[]);
  void displayPrintXY(int x, int y, char s[]);
  void displayPrintXY(int x, int y, const __FlashStringHelper *s);
  void displayXYUInt(uint8_t x, uint8_t y, uint16_t i, uint8_t digits);
  void displayClrLine(uint8_t y);
  void scrollText(char* s, uint8_t y);
  void scrollTextFast(char* s, uint8_t y);
  
  //static const PROGMEM char stretch2bits[] = {
  const char stretch2bits[4] = {
    0x00,
    0x0f,
    0xf0,
    0xff,
  };
  
  //static const PROGMEM char stretch4bits[] = {
  const char stretch4bits[16] = {
    0x00,
    0x03,
    0x0c,
    0x0f,
    0x30,
    0x33,
    0x3c,
    0x3f,
    0xc0,
    0xc3,
    0xcc,
    0xcf,
    0xf0,
    0xf3,
    0xfc,
    0xff,
  };
  
  
  
  #ifdef fnt6x8
    //  
    //  Font.... fnt6x8
    //  charx...  6
    //  chary...  8
    //  
    //  fntdir.. V
    //  fntbit.. LSB
    //  mode.... CHAR
    //  format.. C
    
  #define fontCharacterBytes 6
  //static const PROGMEM uint8_t font[] = {
  const uint8_t font[fontCharacterBytes*96] = {
    0x00,0x00,0x00,0x00,0x00,0x00,  //  Char  ( ) 32
    0x00,0x00,0x5F,0x00,0x00,0x00,  //  Char  (!) 33
    0x00,0x07,0x00,0x07,0x00,0x00,  //  Char  (") 34
    0x14,0x7F,0x14,0x7F,0x14,0x00,  //  Char  (#) 35
    0x24,0x2A,0x7F,0x2A,0x12,0x00,  //  Char  ($) 36
    0x23,0x13,0x08,0x64,0x62,0x00,  //  Char  (%) 37
    0x36,0x49,0x55,0x22,0x50,0x00,  //  Char  (&) 38
    0x00,0x05,0x03,0x00,0x00,0x00,  //  Char  (') 39
    0x00,0x1C,0x22,0x41,0x00,0x00,  //  Char  (() 40
    0x00,0x41,0x22,0x1C,0x00,0x00,  //  Char  ()) 41
    0x14,0x08,0x3E,0x08,0x14,0x00,  //  Char  (*) 42
    0x08,0x08,0x3E,0x08,0x08,0x00,  //  Char  (+) 43
    0x00,0x00,0xB0,0x70,0x00,0x00,  //  Char  (,) 44
    0x08,0x08,0x08,0x08,0x08,0x00,  //  Char  (-) 45
    0x00,0x00,0x60,0x60,0x00,0x00,  //  Char  (.) 46
    0x20,0x10,0x08,0x04,0x02,0x00,  //  Char  (/) 47
  
    0x3E,0x51,0x49,0x45,0x3E,0x00,  //  Char  (0) 48
    0x00,0x42,0x7F,0x40,0x00,0x00,  //  Char  (1) 49
    0x42,0x61,0x51,0x49,0x46,0x00,  //  Char  (2) 50
    0x21,0x41,0x45,0x4B,0x31,0x00,  //  Char  (3) 51
    0x18,0x14,0x12,0x7F,0x10,0x00,  //  Char  (4) 52
    0x27,0x45,0x45,0x45,0x39,0x00,  //  Char  (5) 53
    0x3C,0x4A,0x49,0x49,0x30,0x00,  //  Char  (6) 54
    0x03,0x01,0x71,0x09,0x07,0x00,  //  Char  (7) 55
    0x36,0x49,0x49,0x49,0x36,0x00,  //  Char  (8) 56
    0x06,0x49,0x49,0x29,0x1E,0x00,  //  Char  (9) 57
    0x00,0x6C,0x6C,0x00,0x00,0x00,  //  Char  (:) 58
    0x00,0xAC,0x6C,0x00,0x00,0x00,  //  Char  (;) 59
    0x00,0x08,0x14,0x22,0x41,0x00,  //  Char  (<) 60
    0x14,0x14,0x14,0x14,0x14,0x00,  //  Char  (=) 61
    0x00,0x41,0x22,0x14,0x08,0x00,  //  Char  (>) 62
    0x02,0x01,0x51,0x09,0x06,0x00,  //  Char  (?) 63
  
    0x32,0x49,0x79,0x41,0x3E,0x00,  //  Char  (@) 64
    0x7C,0x12,0x11,0x12,0x7C,0x00,  //  Char  (A) 65
    0x7F,0x49,0x49,0x49,0x36,0x00,  //  Char  (B) 66
    0x3E,0x41,0x41,0x41,0x22,0x00,  //  Char  (C) 67
    0x7F,0x41,0x41,0x22,0x1C,0x00,  //  Char  (D) 68
    0x7F,0x49,0x49,0x41,0x41,0x00,  //  Char  (E) 69
    0x7F,0x09,0x09,0x09,0x01,0x00,  //  Char  (F) 70
    0x3E,0x41,0x49,0x49,0x7A,0x00,  //  Char  (G) 71
    0x7F,0x08,0x08,0x08,0x7F,0x00,  //  Char  (H) 72
    0x00,0x41,0x7F,0x41,0x00,0x00,  //  Char  (I) 73
    0x20,0x40,0x41,0x3F,0x01,0x00,  //  Char  (J) 74
    0x7F,0x08,0x14,0x22,0x41,0x00,  //  Char  (K) 75
    0x7F,0x40,0x40,0x40,0x40,0x00,  //  Char  (L) 76
    0x7F,0x02,0x0C,0x02,0x7F,0x00,  //  Char  (M) 77
    0x7F,0x04,0x08,0x10,0x7F,0x00,  //  Char  (N) 78
    0x3E,0x41,0x41,0x41,0x3E,0x00,  //  Char  (O) 79
  
    0x7F,0x09,0x09,0x09,0x06,0x00,  //  Char  (P) 80
    0x3E,0x41,0x51,0x21,0x5E,0x00,  //  Char  (Q) 81
    0x7F,0x09,0x19,0x29,0x46,0x00,  //  Char  (R) 82
    0x26,0x49,0x49,0x49,0x32,0x00,  //  Char  (S) 83
    0x01,0x01,0x7F,0x01,0x01,0x00,  //  Char  (T) 84
    0x3F,0x40,0x40,0x40,0x3F,0x00,  //  Char  (U) 85
    0x1F,0x20,0x40,0x20,0x1F,0x00,  //  Char  (V) 86
    0x3F,0x40,0x38,0x40,0x3F,0x00,  //  Char  (W) 87
    0x63,0x14,0x08,0x14,0x63,0x00,  //  Char  (X) 88
    0x07,0x08,0x70,0x08,0x07,0x00,  //  Char  (Y) 89
    0x61,0x51,0x49,0x45,0x43,0x00,  //  Char  (Z) 90
    0x00,0x7F,0x41,0x41,0x00,0x00,  //  Char  ([) 91
    0x02,0x04,0x08,0x10,0x20,0x00,  //  Char  (\) 92
    0x00,0x41,0x41,0x7F,0x00,0x00,  //  Char  (]) 93
    0x04,0x02,0x01,0x02,0x04,0x00,  //  Char  (^) 94
    0x80,0x80,0x80,0x80,0x80,0x80,  //  Char  (_) 95
  
    0x00,0x01,0x02,0x04,0x00,0x00,  //  Char  (`) 96
    0x20,0x54,0x54,0x54,0x78,0x00,  //  Char  (a) 97
    0x7F,0x48,0x44,0x44,0x38,0x00,  //  Char  (b) 98
    0x38,0x44,0x44,0x44,0x20,0x00,  //  Char  (c) 99
    0x38,0x44,0x44,0x48,0x7F,0x00,  //  Char  (d) 100
    0x38,0x54,0x54,0x54,0x18,0x00,  //  Char  (e) 101
    0x08,0x7E,0x09,0x01,0x02,0x00,  //  Char  (f) 102
    0x18,0xA4,0xA4,0xA4,0x7C,0x00,  //  Char  (g) 103
    0x7F,0x08,0x04,0x04,0x78,0x00,  //  Char  (h) 104
    0x00,0x48,0x7D,0x40,0x00,0x00,  //  Char  (i) 105
    0x00,0x40,0x80,0x84,0x7D,0x00,  //  Char  (j) 106
    0x00,0x7F,0x10,0x28,0x44,0x00,  //  Char  (k) 107
    0x00,0x41,0x7F,0x40,0x00,0x00,  //  Char  (l) 108
    0x7C,0x04,0x78,0x04,0x78,0x00,  //  Char  (m) 109
    0x7C,0x08,0x04,0x04,0x78,0x00,  //  Char  (n) 110
    0x38,0x44,0x44,0x44,0x38,0x00,  //  Char  (o) 111
  
    0xFC,0x24,0x24,0x24,0x18,0x00,  //  Char  (p) 112
    0x18,0x24,0x24,0x28,0xFC,0x00,  //  Char  (q) 113
    0x7C,0x08,0x04,0x04,0x08,0x00,  //  Char  (r) 114
    0x48,0x54,0x54,0x54,0x20,0x00,  //  Char  (s) 115
    0x04,0x3F,0x44,0x44,0x20,0x00,  //  Char  (t) 116
    0x3C,0x40,0x40,0x20,0x7C,0x00,  //  Char  (u) 117
    0x1C,0x20,0x40,0x20,0x1C,0x00,  //  Char  (v) 118
    0x3C,0x40,0x30,0x40,0x3C,0x00,  //  Char  (w) 119
    0x44,0x28,0x10,0x28,0x44,0x00,  //  Char  (x) 120
    0x1C,0xA0,0xA0,0xA0,0x7C,0x00,  //  Char  (y) 121
    0x44,0x64,0x54,0x4C,0x44,0x00,  //  Char  (z) 122
    0x00,0x08,0x36,0x41,0x00,0x00,  //  Char  ({) 123
    0x00,0x00,0x7F,0x00,0x00,0x00,  //  Char  (|) 124
    0x00,0x41,0x36,0x08,0x00,0x00,  //  Char  (}) 125
    0x08,0x04,0x04,0x08,0x08,0x04,  //  Char  (~) 126
  };
  
  #endif
  
  
  #ifdef fnt8x8
    //  
    //  Font.... fnt8x8
    //  charx...  8
    //  chary...  8
    //  
    //  fntdir.. V
    //  fntbit.. LSB
    //  mode.... CHAR
    //  format.. C
  
  #define fontCharacterBytes 8
  //static const PROGMEM uint8_t font[] = {
  const uint8_t font[fontCharacterBytes*96] = {
  
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,  //  Char  ( ) 32
    0x00,0x00,0x00,0x5F,0x5F,0x00,0x00,0x00,  //  Char  (!) 33
    0x00,0x07,0x07,0x00,0x07,0x07,0x00,0x00,  //  Char  (") 34
    0x14,0x7F,0x7F,0x14,0x7F,0x7F,0x14,0x00,  //  Char  (#) 35
    0x00,0x24,0x2E,0x6B,0x6B,0x3A,0x12,0x00,  //  Char  ($) 36
    0x46,0x66,0x30,0x18,0x0C,0x66,0x62,0x00,  //  Char  (%) 37
    0x30,0x7A,0x4F,0x5D,0x37,0x72,0x48,0x00,  //  Char  (&) 38
    0x00,0x04,0x07,0x03,0x00,0x00,0x00,0x00,  //  Char  (') 39
    0x00,0x00,0x1C,0x3E,0x63,0x41,0x00,0x00,  //  Char  (() 40
    0x00,0x00,0x41,0x63,0x3E,0x1C,0x00,0x00,  //  Char  ()) 41
    0x08,0x2A,0x3E,0x1C,0x1C,0x3E,0x2A,0x08,  //  Char  (*) 42
    0x00,0x08,0x08,0x3E,0x3E,0x08,0x08,0x00,  //  Char  (+) 43
    0x00,0xA0,0xE0,0x60,0x00,0x00,0x00,0x00,  //  Char  (,) 44
    0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x00,  //  Char  (-) 45
    0x00,0x00,0x60,0x60,0x00,0x00,0x00,0x00,  //  Char  (.) 46
    0x60,0x30,0x18,0x0C,0x06,0x03,0x01,0x00,  //  Char  (/) 47
  
    0x00,0x3E,0x7F,0x49,0x45,0x7F,0x3E,0x00,  //  Char  (0) 48
    0x00,0x40,0x42,0x7F,0x7F,0x40,0x40,0x00,  //  Char  (1) 49
    0x00,0x42,0x63,0x71,0x59,0x4F,0x46,0x00,  //  Char  (2) 50
    0x00,0x41,0x41,0x49,0x49,0x7F,0x36,0x00,  //  Char  (3) 51
    0x00,0x18,0x1C,0x16,0x13,0x7F,0x7F,0x00,  //  Char  (4) 52
    0x00,0x27,0x67,0x45,0x45,0x7D,0x39,0x00,  //  Char  (5) 53
    0x00,0x3C,0x7E,0x4B,0x49,0x79,0x30,0x00,  //  Char  (6) 54
    0x00,0x01,0x61,0x71,0x19,0x0F,0x07,0x00,  //  Char  (7) 55
    0x00,0x36,0x7F,0x49,0x49,0x7F,0x36,0x00,  //  Char  (8) 56
    0x00,0x06,0x4F,0x49,0x69,0x3F,0x1E,0x00,  //  Char  (9) 57
    0x00,0x00,0x00,0x6C,0x6C,0x00,0x00,0x00,  //  Char  (:) 58
    0x00,0x00,0xA0,0xEC,0x6C,0x00,0x00,0x00,  //  Char  (;) 59
    0x00,0x08,0x1C,0x36,0x63,0x41,0x00,0x00,  //  Char  (<) 60
    0x00,0x14,0x14,0x14,0x14,0x14,0x14,0x00,  //  Char  (=) 61
    0x00,0x00,0x41,0x63,0x36,0x1C,0x08,0x00,  //  Char  (>) 62
    0x00,0x02,0x03,0x51,0x59,0x0F,0x06,0x00,  //  Char  (?) 63
  
    0x3E,0x7F,0x41,0x5D,0x5D,0x1F,0x1E,0x00,  //  Char  (@) 64
    0x00,0x7C,0x7E,0x13,0x13,0x7E,0x7C,0x00,  //  Char  (A) 65
    0x00,0x7F,0x7F,0x49,0x49,0x7F,0x36,0x00,  //  Char  (B) 66
    0x00,0x1C,0x3E,0x63,0x41,0x41,0x41,0x00,  //  Char  (C) 67
    0x00,0x7F,0x7F,0x41,0x63,0x7F,0x3E,0x00,  //  Char  (D) 68
    0x00,0x7F,0x7F,0x49,0x49,0x41,0x41,0x00,  //  Char  (E) 69
    0x00,0x7F,0x7F,0x09,0x09,0x01,0x01,0x00,  //  Char  (F) 70
    0x00,0x3E,0x7F,0x41,0x49,0x79,0x79,0x00,  //  Char  (G) 71
    0x00,0x7F,0x7F,0x08,0x08,0x7F,0x7F,0x00,  //  Char  (H) 72
    0x00,0x00,0x41,0x7F,0x7F,0x41,0x00,0x00,  //  Char  (I) 73
    0x00,0x20,0x60,0x40,0x40,0x7F,0x3F,0x00,  //  Char  (J) 74
    0x00,0x7F,0x7F,0x08,0x1C,0x77,0x63,0x00,  //  Char  (K) 75
    0x00,0x7F,0x7F,0x40,0x40,0x40,0x40,0x00,  //  Char  (L) 76
    0x7F,0x7F,0x06,0x0C,0x06,0x7F,0x7F,0x00,  //  Char  (M) 77
    0x00,0x7F,0x7F,0x06,0x0C,0x7F,0x7F,0x00,  //  Char  (N) 78
    0x00,0x3E,0x7F,0x41,0x41,0x7F,0x3E,0x00,  //  Char  (O) 79
  
    0x00,0x7F,0x7F,0x09,0x09,0x0F,0x06,0x00,  //  Char  (P) 80
    0x00,0x1E,0x3F,0x21,0x71,0x7F,0x5E,0x00,  //  Char  (Q) 81
    0x00,0x7F,0x7F,0x19,0x39,0x6F,0x46,0x00,  //  Char  (R) 82
    0x00,0x46,0x4F,0x4D,0x59,0x79,0x31,0x00,  //  Char  (S) 83
    0x00,0x01,0x01,0x7F,0x7F,0x01,0x01,0x00,  //  Char  (T) 84
    0x00,0x3F,0x7F,0x40,0x40,0x7F,0x7F,0x00,  //  Char  (U) 85
    0x00,0x1F,0x3F,0x60,0x60,0x3F,0x1F,0x00,  //  Char  (V) 86
    0x7F,0x7F,0x30,0x18,0x30,0x7F,0x7F,0x00,  //  Char  (W) 87
    0x00,0x63,0x77,0x1C,0x1C,0x77,0x63,0x00,  //  Char  (X) 88
    0x00,0x07,0x0F,0x78,0x78,0x0F,0x07,0x00,  //  Char  (Y) 89
    0x00,0x61,0x71,0x59,0x4D,0x47,0x43,0x00,  //  Char  (Z) 90
    0x00,0x00,0x7F,0x7F,0x41,0x41,0x00,0x00,  //  Char  ([) 91
    0x01,0x03,0x06,0x0C,0x18,0x30,0x60,0x00,  //  Char  (\) 92
    0x00,0x00,0x41,0x41,0x7F,0x7F,0x00,0x00,  //  Char  (]) 93
    0x08,0x0C,0x06,0x03,0x06,0x0C,0x08,0x00,  //  Char  (^) 94
    0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x00,  //  Char  (_) 95
  
    0x00,0x00,0x03,0x07,0x04,0x00,0x00,0x00,  //  Char  (`) 96
    0x00,0x38,0x7C,0x44,0x44,0x7C,0x7C,0x00,  //  Char  (a) 97
    0x00,0x7F,0x7F,0x44,0x44,0x7C,0x38,0x00,  //  Char  (b) 98
    0x00,0x38,0x7C,0x44,0x44,0x44,0x44,0x00,  //  Char  (c) 99
    0x00,0x30,0x78,0x48,0x48,0x7F,0x7F,0x00,  //  Char  (d) 100
    0x00,0x38,0x7C,0x54,0x54,0x5C,0x18,0x00,  //  Char  (e) 101
    0x00,0x00,0x7E,0x7F,0x09,0x01,0x00,0x00,  //  Char  (f) 102
    0x00,0x98,0xBC,0xA4,0xA4,0xFC,0x7C,0x00,  //  Char  (g) 103
    0x00,0x7F,0x7F,0x04,0x04,0x7C,0x78,0x00,  //  Char  (h) 104
    0x00,0x00,0x44,0x7D,0x7D,0x40,0x00,0x00,  //  Char  (i) 105
    0x00,0x80,0x80,0xFD,0x7D,0x00,0x00,0x00,  //  Char  (j) 106
    0x00,0x7F,0x7F,0x10,0x38,0x6C,0x44,0x00,  //  Char  (k) 107
    0x00,0x00,0x41,0x7F,0x7F,0x40,0x00,0x00,  //  Char  (l) 108
    0x7C,0x7C,0x0C,0x18,0x0C,0x7C,0x78,0x00,  //  Char  (m) 109
    0x00,0x7C,0x7C,0x04,0x04,0x7C,0x78,0x00,  //  Char  (n) 110
    0x00,0x38,0x7C,0x44,0x44,0x7C,0x38,0x00,  //  Char  (o) 111
  
    0x00,0xFC,0xFC,0x24,0x24,0x3C,0x18,0x00,  //  Char  (p) 112
    0x00,0x18,0x3C,0x24,0x24,0xFC,0xFC,0x00,  //  Char  (q) 113
    0x00,0x7C,0x7C,0x08,0x04,0x04,0x04,0x00,  //  Char  (r) 114
    0x00,0x48,0x5C,0x54,0x54,0x74,0x24,0x00,  //  Char  (s) 115
    0x00,0x04,0x3F,0x7F,0x44,0x44,0x00,0x00,  //  Char  (t) 116
    0x00,0x3C,0x7C,0x40,0x40,0x7C,0x7C,0x00,  //  Char  (u) 117
    0x00,0x1C,0x3C,0x60,0x60,0x3C,0x1C,0x00,  //  Char  (v) 118
    0x3C,0x7C,0x60,0x30,0x60,0x7C,0x3C,0x00,  //  Char  (w) 119
    0x00,0x44,0x6C,0x38,0x38,0x6C,0x44,0x00,  //  Char  (x) 120
    0x00,0x9C,0xBC,0xA0,0xA0,0xFC,0x7C,0x00,  //  Char  (y) 121
    0x00,0x44,0x64,0x74,0x5C,0x4C,0x44,0x00,  //  Char  (z) 122
    0x00,0x08,0x08,0x3E,0x77,0x41,0x41,0x00,  //  Char  ({) 123
    0x00,0x00,0x00,0x77,0x77,0x00,0x00,0x00,  //  Char  (|) 124
    0x00,0x41,0x41,0x77,0x3E,0x08,0x08,0x00,  //  Char  (}) 125
    0x02,0x03,0x01,0x03,0x02,0x03,0x01,0x00,  //  Char  (~) 126
  
  };
  
  #endif  
};


// Fundamental Command
// (00h~0Fh) Set Lower Column Start Address for Page Addressing Mode
#define D1306_SETLOWCOLUMN 0x00
// (10h~1Fh) Set Higher Column Start Address for Page Addressing Mode
#define D1306_SETHIGHCOLUMN 0x10
// (20h, addr) Set Memory Addressing Mode
#define D1306_MEMORYMODE 0x20
// (21h, Start, End) Set Column Address
#define D1306_COLUMNADDR 0x21
// (22h, Start, End) Set Page Address
#define D1306_PAGEADDR   0x22
// (40h~7Fh) Set Display Start Line
#define D1306_STARTLINE 0x40
// (81h, value) Set Contrast Control for BANK0
#define D1306_CONTRAST 0x81
// (8Dh, value) Charge Pump Setting
#define D1306_CHARGEPUMP 0x8D
// (A0h/A1h) Set Segment Re-map
#define D1306_SEGREMAP 0xA0
// (A4h/A5h) Entire Display OFF/ON
#define D1306_ENTIREDISPON 0xA4
// (A6h/A7h) Set Display Normal/Inverse
#define D1306_INVERSE  0xA6
// (A8h) Set Multiplex Ratio
#define D1306_MUXRATIO 0xA8
// (AEh/AFh) Set Display ON/OFF
#define D1306_DISPLAYON 0xAE
// (B0h~B7h) Set Page Start Address for Page Addressing Mode
#define D1306_PGSTARTADDR 0xB0
// (C0h/C8h) Set COM Output Scan Direction
#define D1306_COMSCAN 0xC0
#define D1306_COMSCANVERTFLIP 0xC8
// (D3h) Set Display Offset
#define D1306_DISPLAYOFFSET 0xD3
// (D5h) Set Display Clock Divide Ratio/Oscillator Frequency
#define D1306_DISPLAYCLKDIV 0xD5
// (D9h) Set Pre-charge Period
#define D1306_PRECHARGEPERIOD 0xD9
// (DAh) Set COM Pins Hardware Configuration
#define D1306_COMPINSHWCONFIG 0xDA
// (DBh) Set VCOMH Deselect Level, adjust regulator output
#define D1306_VCOMHADJUST 0xDB
// (E3h) NOP
#define D1306_NOP 0xE3

// Graphic Acceleration Command (actually not used)
// Horizontal Scroll Setup (26h/27h)
// Continuous Vertical and Horizontal Scroll Setup (29h/2Ah)
// Deactivate Scroll (2Eh)
// Activate Scroll (2Fh)
// Set Vertical Scroll Area(A3h)

#endif

